@@ -94,7 +94,11 @@ slot_print(Cache8 c, sid_t key, unsigned int depth) {
 	    k = (k8 << BITS) | i;
 	    /*printf("*** key: 0x%016llx  depth: %u  i: %u\n", k, depth, i); */
 	    if (DEPTH - 1 == depth) {
+#if IS_WINDOWS
+		printf("0x%016lx: %4lu\n", (long unsigned int)k, (long unsigned int)b->value);
+#else
 		printf("0x%016llx: %4llu\n", (long long unsigned int)k, (long long unsigned int)b->value);
+#endif
 	    } else {
 		slot_print(b->child, k, depth + 1);
 	    }
