@@ -32,6 +32,10 @@
 #define __OJ_ERR_H__
 
 #include "ruby.h"
+// Needed to silence 2.4.0 warnings.
+#ifndef NORETURN
+# define NORETURN(x) x
+#endif
 
 #define set_error(err, eclas, msg, json, current) _oj_err_set_with_location(err, eclas, msg, json, current, __FILE__, __LINE__)
 
@@ -44,10 +48,12 @@ extern VALUE	oj_parse_error_class;
 
 extern void	oj_err_set(Err e, VALUE clas, const char *format, ...);
 extern void	_oj_err_set_with_location(Err err, VALUE eclas, const char *msg, const char *json, const char *current, const char* file, int line);
-extern void	oj_err_raise(Err e);
+
+NORETURN(extern void	oj_err_raise(Err e));
 
 #define raise_error(msg, json, current) _oj_raise_error(msg, json, current, __FILE__, __LINE__)
-extern void	_oj_raise_error(const char *msg, const char *json, const char *current, const char* file, int line);
+
+NORETURN(extern void	_oj_raise_error(const char *msg, const char *json, const char *current, const char* file, int line));
 
 
 inline static void
