@@ -32,8 +32,11 @@ dflags = {
   'IS_WINDOWS' => is_windows ? 1 : 0,
   'USE_PTHREAD_MUTEX' => is_windows ? 0 : 1,
   'USE_RB_MUTEX' => (is_windows && !('1' == version[0] && '8' == version[1])) ? 1 : 0,
-  'DATETIME_1_8' => ('ruby' == type && ('1' == version[0] && '8' == version[1])) ? 1 : 0,
   'NO_TIME_ROUND_PAD' => ('rubinius' == type) ? 1 : 0,
+  'HAS_DATA_OBJECT_WRAP' => ('ruby' == type && '2' == version[0] && '3' <= version[1]) ? 1 : 0,
+  'HAS_METHOD_ARITY' =>  ('rubinius' == type) ? 0 : 1,
+  'HAS_STRUCT_MEMBERS' =>  ('rubinius' == type) ? 0 : 1,
+  'RSTRUCT_LEN_RETURNS_INTEGER_OBJECT' => ('ruby' == type && '2' == version[0] && '4' == version[1] && '1' >= version[2]) ? 1 : 0,
 }
 # This is a monster hack to get around issues with 1.9.3-p0 on CentOS 5.4. SO
 # some reason math.h and string.h contents are not processed. Might be a
@@ -47,6 +50,8 @@ else
   dflags['NEEDS_STPCPY'] = nil if is_windows
 end
 
+dflags['OJ_DEBUG'] = true unless ENV['OJ_DEBUG'].nil?
+
 dflags.each do |k,v|
   if v.nil?
     $CPPFLAGS += " -D#{k}"
@@ -54,8 +59,9 @@ dflags.each do |k,v|
     $CPPFLAGS += " -D#{k}=#{v}"
   end
 end
+
 $CPPFLAGS += ' -Wall'
 #puts "*** $CPPFLAGS: #{$CPPFLAGS}"
-create_makefile(extension_name)
+create_makefile(File.join(extension_name, extension_name))
 
-%x{make clean}
+#%x{make clean}
