@@ -92,7 +92,7 @@ static char*	read_quoted_value(ParseInfo pi);
 static void	skip_comment(ParseInfo pi);
 
 static VALUE	protect_open_proc(VALUE x);
-static VALUE	parse_json(VALUE clas, char *json, int given, int allocated);
+static VALUE	parse_json(VALUE clas, char *json, bool given, bool allocated);
 static void	each_leaf(Doc doc, VALUE self);
 static int	move_step(Doc doc, const char *path, int loc);
 static Leaf	get_doc_leaf(Doc doc, const char *path);
@@ -779,7 +779,7 @@ free_doc_cb(void *x) {
 }
 
 static VALUE
-parse_json(VALUE clas, char *json, int given, int allocated) {
+parse_json(VALUE clas, char *json, bool given, bool allocated) {
     struct _ParseInfo	pi;
     VALUE		result = Qnil;
     Doc			doc;
@@ -867,6 +867,35 @@ get_doc_leaf(Doc doc, const char *path) {
     return leaf;
 }
 
+static const char*
+next_slash(const char *s) {
+    for (; '\0' != *s; s++) {
+	if ('\\' == *s) {
+	    s++;
+	    if ('\0' == *s) {
+		break;
+	    }
+	} else if ('/' == *s) {
+	    return s;
+	}
+    }
+    return NULL;
+}
+
+static bool
+key_match(const char *pat, const char *key, int plen) {
+    for (; 0 < plen; plen--, pat++, key++) {
+	if ('\\' == *pat) {
+	    plen--;
+	    pat++;
+	}
+	if (*pat != *key) {
+	    return false;
+	}
+    }
+    return '\0' == *key;
+}
+
 static Leaf
 get_leaf(Leaf *stack, Leaf *lp, const char *path) {
     Leaf	leaf = *lp;
@@ -912,7 +941,7 @@ get_leaf(Leaf *stack, Leaf *lp, const char *path) {
 		} while (e != first);
 	    } else if (T_HASH == type) {
 		const char	*key = path;
-		const char	*slash = strchr(path, '/');
+		const char	*slash = next_slash(path);
 		int		klen;
 
 		if (0 == slash) {
@@ -923,7 +952,7 @@ get_leaf(Leaf *stack, Leaf *lp, const char *path) {
 		    path += klen + 1;
 		}
 		do {
-		    if (0 == strncmp(key, e->key, klen) && '\0' == e->key[klen]) {
+		    if (key_match(key, e->key, klen)) {
 			lp++;
 			*lp = e;
 			leaf = get_leaf(stack, lp, path);
@@ -1022,7 +1051,7 @@ move_step(Doc doc, const char *path, int loc) {
 		} while (e != first);
 	    } else if (T_HASH == leaf->rtype) {
 		const char	*key = path;
-		const char	*slash = strchr(path, '/');
+		const char	*slash = next_slash(path);
 		int		klen;
 
 		if (0 == slash) {
@@ -1033,7 +1062,7 @@ move_step(Doc doc, const char *path, int loc) {
 		    path += klen + 1;
 		}
 		do {
-		    if (0 == strncmp(key, e->key, klen) && '\0' == e->key[klen]) {
+		    if (key_match(key, e->key, klen)) {
 			doc->where++;
 			*doc->where = e;
 			loc = move_step(doc, path, loc + 1);
@@ -1070,14 +1099,14 @@ each_value(Doc doc, Leaf leaf) {
 
 // doc functions
 
-/* call-seq: open(json) { |doc| ... } => Object
+/* @overload open(json) { |doc| ... } => Object
  *
  * Parses a JSON document String and then yields to the provided block if one
  * is given with an instance of the Oj::Doc as the single yield parameter. If
  * a block is not given then an Oj::Doc instance is returned and must be
  * closed with a call to the #close() method when no longer needed.
  *
- * @param [String] json JSON document string
+ *   @param [String] json JSON document string
  * @yieldparam [Oj::Doc] doc parsed JSON document
  * @yieldreturn [Object] returns the result of the yield as the result of the method call
  * @example
@@ -1111,14 +1140,14 @@ doc_open(VALUE clas, VALUE str) {
     return obj;
 }
 
-/* call-seq: open_file(filename) { |doc| ... } => Object
+/* @overload open_file(filename) { |doc| ... } => Object
  *
  * Parses a JSON document from a file and then yields to the provided block if
  * one is given with an instance of the Oj::Doc as the single yield
  * parameter. If a block is not given then an Oj::Doc instance is returned and
  * must be closed with a call to the #close() method when no longer needed.
  *
- * @param [String] filename name of file that contains a JSON document
+ *   @param [String] filename name of file that contains a JSON document
  * @yieldparam [Oj::Doc] doc parsed JSON document
  * @yieldreturn [Object] returns the result of the yield as the result of the method call
  * @example
@@ -1167,11 +1196,34 @@ doc_open_file(VALUE clas, VALUE filename) {
     return obj;
 }
 
+static int
+esc_strlen(const char *s) {
+    int	cnt = 0;
+
+    for (; '\0' != *s; s++, cnt++) {
+	if ('/' == *s) {
+	    cnt++;
+	}
+    }
+    return cnt;
+}
+
+static char*
+append_key(char *p, const char *key) {
+    for (; '\0' != *key; p++, key++) {
+	if ('/' == *key) {
+	    *p++ = '\\';
+	}
+	*p = *key;
+    }
+    return p;
+}
+
 /* Document-method: parse
  * @see Oj::Doc.open
  */
 
-/* call-seq: where?() => String
+/* @overload where?() => String
  *
  * Returns a String that describes the absolute path to the current location
  * in the JSON document.
@@ -1192,7 +1244,7 @@ doc_where(VALUE self) {
 	for (lp = doc->where_path; lp <= doc->where; lp++) {
 	    leaf = *lp;
 	    if (T_HASH == leaf->parent_type) {
-		size += strlen((*lp)->key) + 1;
+		size += esc_strlen((*lp)->key) + 1;
 	    } else if (T_ARRAY == leaf->parent_type) {
 		size += ((*lp)->index < 100) ? 3 : 11;
 	    }
@@ -1202,7 +1254,7 @@ doc_where(VALUE self) {
 	for (lp = doc->where_path; lp <= doc->where; lp++) {
 	    leaf = *lp;
 	    if (T_HASH == leaf->parent_type) {
-		p = stpcpy(p, (*lp)->key);
+		p = append_key(p, (*lp)->key);
 	    } else if (T_ARRAY == leaf->parent_type) {
 		p = ulong_fill(p, (*lp)->index);
 	    }
@@ -1213,7 +1265,7 @@ doc_where(VALUE self) {
     }
 }
 
-/* call-seq: local_key() => String, Fixnum, nil
+/* @overload local_key() => String, Fixnum, nil
  *
  * Returns the final key to the current location.
  * @example
@@ -1236,7 +1288,7 @@ doc_local_key(VALUE self) {
     return key;
 }
 
-/* call-seq: home() => nil
+/* @overload home() => nil
  *
  * Moves the document marker or location to the hoot or home position. The
  * same operation can be performed with a Oj::Doc.move('/').
@@ -1253,13 +1305,13 @@ doc_home(VALUE self) {
     return oj_slash_string;
 }
 
-/* call-seq: type(path=nil) => Class
+/* @overload type(path=nil) => Class
  *
  * Returns the Class of the data value at the location identified by the path
  * or the current location if the path is nil or not provided. This method
  * does not create the Ruby Object at the location specified so the overhead
  * is low.
- * @param [String] path path to the location to get the type of if provided
+ *   @param [String] path path to the location to get the type of if provided
  * @example
  *   Oj::Doc.open('[1,2]') { |doc| doc.type() }	     #=> Array
  *   Oj::Doc.open('[1,2]') { |doc| doc.type('/1') }  #=> Fixnum
@@ -1295,14 +1347,14 @@ doc_type(int argc, VALUE *argv, VALUE self) {
     return type;
 }
 
-/* call-seq: fetch(path=nil) => nil, true, false, Fixnum, Float, String, Array, Hash
+/* @overload fetch(path=nil) => nil, true, false, Fixnum, Float, String, Array, Hash
  *
  * Returns the value at the location identified by the path or the current
  * location if the path is nil or not provided. This method will create and
  * return an Array or Hash if that is the type of Object at the location
  * specified. This is more expensive than navigating to the leaves of the JSON
  * document.
- * @param [String] path path to the location to get the type of if provided
+ *   @param [String] path path to the location to get the type of if provided
  * @example
  *   Oj::Doc.open('[1,2]') { |doc| doc.fetch() }      #=> [1, 2]
  *   Oj::Doc.open('[1,2]') { |doc| doc.fetch('/1') }  #=> 1
@@ -1328,12 +1380,12 @@ doc_fetch(int argc, VALUE *argv, VALUE self) {
     return val;
 }
 
-/* call-seq: each_leaf(path=nil) => nil
+/* @overload each_leaf(path=nil) => nil
  *
  * Yields to the provided block for each leaf node with the identified
  * location of the JSON document as the root. The parameter passed to the
  * block on yield is the Doc instance after moving to the child location.
- * @param [String] path if provided it identified the top of the branch to process the leaves of
+ *   @param [String] path if provided it identified the top of the branch to process the leaves of
  * @yieldparam [Doc] Doc at the child location
  * @example
  *   Oj::Doc.open('[3,[2,1]]') { |doc|
@@ -1377,11 +1429,11 @@ doc_each_leaf(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
-/* call-seq: move(path) => nil
+/* @overload move(path) => nil
  *
  * Moves the document marker to the path specified. The path can an absolute
  * path or a relative path.
- * @param [String] path path to the location to move to
+ *   @param [String] path path to the location to move to
  * @example
  *   Oj::Doc.open('{"one":[1,2]') { |doc| doc.move('/one/2'); doc.where? }  #=> "/one/2"
  */
@@ -1403,13 +1455,13 @@ doc_move(VALUE self, VALUE str) {
     return Qnil;
 }
 
-/* call-seq: each_child(path=nil) { |doc| ... } => nil
+/* @overload each_child(path=nil) { |doc| ... } => nil
  *
  * Yields to the provided block for each immediate child node with the
  * identified location of the JSON document as the root. The parameter passed
  * to the block on yield is the Doc instance after moving to the child
  * location.
- * @param [String] path if provided it identified the top of the branch to process the chilren of
+ *   @param [String] path if provided it identified the top of the branch to process the chilren of
  * @yieldparam [Doc] Doc at the child location
  * @example
  *   Oj::Doc.open('[3,[2,1]]') { |doc|
@@ -1463,13 +1515,13 @@ doc_each_child(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
-/* call-seq: each_value(path=nil) { |val| ... } => nil
+/* @overload each_value(path=nil) { |val| ... } => nil
  *
  * Yields to the provided block for each leaf value in the identified location
  * of the JSON document. The parameter passed to the block on yield is the
  * value of the leaf. Only those leaves below the element specified by the
  * path parameter are processed.
- * @param [String] path if provided it identified the top of the branch to process the leaf values of
+ *   @param [String] path if provided it identified the top of the branch to process the leaf values of
  * @yieldparam [Object] val each leaf value
  * @example
  *   Oj::Doc.open('[3,[2,1]]') { |doc|
@@ -1504,12 +1556,12 @@ doc_each_value(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
-/* call-seq: dump(path=nil) => String
+/* @overload dump(path, filename)
  *
  * Dumps the document or nodes to a new JSON document. It uses the default
  * options for generating the JSON.
- * @param [String] path if provided it identified the top of the branch to dump to JSON
- * @param [String] filename if provided it is the filename to write the output to
+ *   @param path [String] if provided it identified the top of the branch to dump to JSON
+ *   @param filename [String] if provided it is the filename to write the output to
  * @example
  *   Oj::Doc.open('[3,[2,1]]') { |doc|
  *       doc.dump('/2')
@@ -1542,7 +1594,7 @@ doc_dump(int argc, VALUE *argv, VALUE self) {
 
 	    out.buf = buf;
 	    out.end = buf + sizeof(buf) - 10;
-	    out.allocated = 0;
+	    out.allocated = false;
 	    out.omit_nil = oj_default_options.dump_opts.omit_nil;
 	    oj_dump_leaf_to_json(leaf, &oj_default_options, &out);
 	    rjson = rb_str_new2(out.buf);
@@ -1558,7 +1610,7 @@ doc_dump(int argc, VALUE *argv, VALUE self) {
     return Qnil;
 }
 
-/* call-seq: size() => Fixnum
+/* @overload size() => Fixnum
  *
  * Returns the number of nodes in the JSON document where a node is any one of
  * the basic JSON components.
@@ -1571,7 +1623,7 @@ doc_size(VALUE self) {
     return ULONG2NUM(((Doc)DATA_PTR(self))->size);
 }
 
-/* call-seq: close() => nil
+/* @overload close() => nil
  *
  * Closes an open document. No further calls to the document will be valid
  * after closing.
